@@ -1,5 +1,5 @@
 /* chgrp -- change group ownership of files
-   Copyright (C) 1989-2020 Free Software Foundation, Inc.
+   Copyright (C) 1989-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ static struct option const long_options[] =
 /* Return the group ID of NAME, or -1 if no name was specified.  */
 
 static gid_t
-parse_group (const char *name)
+parse_group (char const *name)
 {
   gid_t gid = -1;
 
