@@ -1,5 +1,5 @@
 /* dd -- convert a file while copying it.
-   Copyright (C) 1985-2020 Free Software Foundation, Inc.
+   Copyright (C) 1985-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -246,10 +246,14 @@ static uintmax_t r_truncate = 0;
 static char newline_character = '\n';
 static char space_character = ' ';
 
-/* Input buffer. */
-static char *ibuf;
+#ifdef lint
+/* Memory blocks allocated for I/O buffers and surrounding areas.  */
+static char *real_ibuf;
+static char *real_obuf;
+#endif
 
-/* Output buffer. */
+/* I/O buffers.  */
+static char *ibuf;
 static char *obuf;
 
 /* Current index into 'obuf'. */
@@ -536,7 +540,7 @@ maybe_close_stdout (void)
 /* Like the 'error' function but handle any pending newline.  */
 
 static void _GL_ATTRIBUTE_FORMAT ((__printf__, 3, 4))
-nl_error (int status, int errnum, const char *fmt, ...)
+nl_error (int status, int errnum, char const *fmt, ...)
 {
   if (0 < progress_len)
     {
@@ -697,8 +701,8 @@ alloc_ibuf (void)
   if (ibuf)
     return;
 
-  char *real_buf = malloc (input_blocksize + INPUT_BLOCK_SLOP);
-  if (!real_buf)
+  char *buf = malloc (input_blocksize + INPUT_BLOCK_SLOP);
+  if (!buf)
     {
       uintmax_t ibs = input_blocksize;
       char hbuf[LONGEST_HUMAN_READABLE + 1];
@@ -708,10 +712,10 @@ alloc_ibuf (void)
            human_readable (input_blocksize, hbuf,
                            human_opts | human_base_1024, 1, 1));
     }
-
-  real_buf += SWAB_ALIGN_OFFSET;	/* allow space for swab */
-
-  ibuf = ptr_align (real_buf, page_size);
+#ifdef lint
+  real_ibuf = buf;
+#endif
+  ibuf = ptr_align (buf + SWAB_ALIGN_OFFSET, page_size);
 }
 
 /* Ensure output buffer OBUF is allocated/initialized.  */
@@ -725,8 +729,8 @@ alloc_obuf (void)
   if (conversions_mask & C_TWOBUFS)
     {
       /* Page-align the output buffer, too.  */
-      char *real_obuf = malloc (output_blocksize + OUTPUT_BLOCK_SLOP);
-      if (!real_obuf)
+      char *buf = malloc (output_blocksize + OUTPUT_BLOCK_SLOP);
+      if (!buf)
         {
           uintmax_t obs = output_blocksize;
           char hbuf[LONGEST_HUMAN_READABLE + 1];
@@ -737,7 +741,10 @@ alloc_obuf (void)
                human_readable (output_blocksize, hbuf,
                                human_opts | human_base_1024, 1, 1));
         }
-      obuf = ptr_align (real_obuf, page_size);
+#ifdef lint
+      real_obuf = buf;
+#endif
+      obuf = ptr_align (buf, page_size);
     }
   else
     {
@@ -966,6 +973,13 @@ iclose (int fd)
 static void
 cleanup (void)
 {
+#ifdef lint
+  free (real_ibuf);
+  free (real_obuf);
+  real_ibuf = NULL;
+  real_obuf = NULL;
+#endif
+
   if (iclose (STDIN_FILENO) != 0)
     die (EXIT_FAILURE, errno, _("closing input file %s"), quoteaf (input_file));
 
@@ -1027,7 +1041,7 @@ quit (int code)
 }
 
 /* Return LEN rounded down to a multiple of IO_BUFSIZE
-   (to minimize calls to the expensive posix_fadvise(,POSIX_FADV_DONTNEED),
+   (to minimize calls to the expensive posix_fadvise (,POSIX_FADV_DONTNEED),
    while storing the remainder internally per FD.
    Pass LEN == 0 to get the current remainder.  */
 
@@ -1220,7 +1234,7 @@ iwrite (int fd, char const *buf, size_t size)
       invalidate_cache (STDOUT_FILENO, 0);
 
       /* Attempt to ensure that that final block is committed
-         to disk as quickly as possible.  */
+         to stable storage as quickly as possible.  */
       conversions_mask |= C_FSYNC;
 
       /* After the subsequent fsync we'll call invalidate_cache
@@ -1435,7 +1449,7 @@ parse_symbols (char const *str, struct symbol_value const *table,
    number in this format.  */
 
 static uintmax_t
-parse_integer (const char *str, strtol_error *invalid)
+parse_integer (char const *str, strtol_error *invalid)
 {
   uintmax_t n;
   char *suffix;
@@ -1774,59 +1788,6 @@ advance_input_offset (uintmax_t offset)
     input_offset_overflow = true;
 }
 
-/* This is a wrapper for lseek.  It detects and warns about a kernel
-   bug that makes lseek a no-op for tape devices, even though the kernel
-   lseek return value suggests that the function succeeded.
-
-   The parameters are the same as those of the lseek function, but
-   with the addition of FILENAME, the name of the file associated with
-   descriptor FDESC.  The file name is used solely in the warning that's
-   printed when the bug is detected.  Return the same value that lseek
-   would have returned, but when the lseek bug is detected, return -1
-   to indicate that lseek failed.
-
-   The offending behavior has been confirmed with an Exabyte SCSI tape
-   drive accessed via /dev/nst0 on both Linux 2.2.17 and 2.4.16 kernels.  */
-
-#if defined __linux__ && HAVE_SYS_MTIO_H
-
-# include <sys/mtio.h>
-
-# define MT_SAME_POSITION(P, Q) \
-   ((P).mt_resid == (Q).mt_resid \
-    && (P).mt_fileno == (Q).mt_fileno \
-    && (P).mt_blkno == (Q).mt_blkno)
-
-static off_t
-skip_via_lseek (char const *filename, int fdesc, off_t offset, int whence)
-{
-  struct mtget s1;
-  struct mtget s2;
-  bool got_original_tape_position = (ioctl (fdesc, MTIOCGET, &s1) == 0);
-  /* known bad device type */
-  /* && s.mt_type == MT_ISSCSI2 */
-
-  off_t new_position = lseek (fdesc, offset, whence);
-  if (0 <= new_position
-      && got_original_tape_position
-      && ioctl (fdesc, MTIOCGET, &s2) == 0
-      && MT_SAME_POSITION (s1, s2))
-    {
-      if (status_level != STATUS_NONE)
-        error (0, 0, _("warning: working around lseek kernel bug for file "
-                       "(%s)\n  of mt_type=0x%0lx -- "
-                       "see <sys/mtio.h> for the list of types"),
-               filename, s2.mt_type + 0Lu);
-      errno = 0;
-      new_position = -1;
-    }
-
-  return new_position;
-}
-#else
-# define skip_via_lseek(Filename, Fd, Offset, Whence) lseek (Fd, Offset, Whence)
-#endif
-
 /* Throw away RECORDS blocks of BLOCKSIZE bytes plus BYTES bytes on
    file descriptor FDESC, which is open with read permission for FILE.
    Store up to BLOCKSIZE bytes of the data at a time in IBUF or OBUF, if
@@ -1848,7 +1809,7 @@ skip (int fdesc, char const *file, uintmax_t records, size_t blocksize,
 
   errno = 0;
   if (records <= OFF_T_MAX / blocksize
-      && 0 <= skip_via_lseek (file, fdesc, offset, SEEK_CUR))
+      && 0 <= lseek (fdesc, offset, SEEK_CUR))
     {
       if (fdesc == STDIN_FILENO)
         {
@@ -1883,7 +1844,7 @@ skip (int fdesc, char const *file, uintmax_t records, size_t blocksize,
          Or it may not have been done at all (> OFF_T_MAX).
          Therefore try to seek to the end of the file,
          to avoid redundant reading.  */
-      if ((skip_via_lseek (file, fdesc, 0, SEEK_END)) >= 0)
+      if (lseek (fdesc, 0, SEEK_END) >= 0)
         {
           /* File is seekable, and we're at the end of it, and
              size <= OFF_T_MAX. So there's no point using read to advance.  */
@@ -1988,7 +1949,7 @@ advance_input_after_read_error (size_t nbytes)
           diff = input_offset - offset;
           if (! (0 <= diff && diff <= nbytes) && status_level != STATUS_NONE)
             error (0, 0, _("warning: invalid file offset after failed read"));
-          if (0 <= skip_via_lseek (input_file, STDIN_FILENO, diff, SEEK_CUR))
+          if (0 <= lseek (STDIN_FILENO, diff, SEEK_CUR))
             return true;
           if (errno == 0)
             error (0, 0, _("cannot work around kernel bug after all"));
@@ -2004,7 +1965,7 @@ advance_input_after_read_error (size_t nbytes)
 static void
 copy_simple (char const *buf, size_t nread)
 {
-  const char *start = buf;	/* First uncopied char in BUF.  */
+  char const *start = buf;	/* First uncopied char in BUF.  */
 
   do
     {
@@ -2162,7 +2123,7 @@ dd_copy (void)
      extra two bytes.  */
 
   /* Some devices require alignment on a sector or page boundary
-     (e.g. character disk devices).  Align the input buffer to a
+     (e.g. character flash or disk devices).  Align the input buffer to a
      page boundary to cover all bases.  Note that due to the swab
      algorithm, we must have at least one byte in the page before
      the input buffer;  thus we allocate 2 pages of slop in the
@@ -2170,8 +2131,8 @@ dd_copy (void)
 
      The page alignment is necessary on any Linux kernel that supports
      either the SGI raw I/O patch or Steven Tweedies raw I/O patch.
-     It is necessary when accessing raw (i.e., character special) disk
-     devices on Unixware or other SVR4-derived system.  */
+     It is necessary when accessing raw (i.e., character special)
+     storage devices on SVR4-derived systems.  */
 
   if (skip_records != 0 || skip_bytes != 0)
     {
@@ -2229,7 +2190,7 @@ dd_copy (void)
   alloc_ibuf ();
   alloc_obuf ();
 
-  while (1)
+  while (true)
     {
       if (status_level == STATUS_PROGRESS)
         {
