@@ -1,5 +1,5 @@
 /* dircolors - output commands to set the LS_COLOR environment variable
-   Copyright (C) 1996-2020 Free Software Foundation, Inc.
+   Copyright (C) 1996-2021 Free Software Foundation, Inc.
    Copyright (C) 1994, 1995, 1997, 1998, 1999, 2000 H. Peter Anvin
 
    This program is free software: you can redistribute it and/or modify
@@ -29,7 +29,6 @@
 #include "obstack.h"
 #include "quote.h"
 #include "stdio--.h"
-#include "xstrndup.h"
 
 /* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "dircolors"
@@ -59,7 +58,7 @@ enum Shell_syntax
    variable.  */
 static struct obstack lsc_obstack;
 
-static const char *const slack_codes[] =
+static char const *const slack_codes[] =
 {
   "NORMAL", "NORM", "FILE", "RESET", "DIR", "LNK", "LINK",
   "SYMLINK", "ORPHAN", "MISSING", "FIFO", "PIPE", "SOCK", "BLK", "BLOCK",
@@ -69,7 +68,7 @@ static const char *const slack_codes[] =
   "MULTIHARDLINK", "CLRTOEOL", NULL
 };
 
-static const char *const ls_codes[] =
+static char const *const ls_codes[] =
 {
   "no", "no", "fi", "rs", "di", "ln", "ln", "ln", "or", "mi", "pi", "pi",
   "so", "bd", "bd", "cd", "cd", "do", "ex", "lc", "lc", "rc", "rc", "ec", "ec",
@@ -163,7 +162,7 @@ parse_line (char const *line, char **keyword, char **arg)
       ++p;
     }
 
-  *keyword = xstrndup (keyword_start, p - keyword_start);
+  *keyword = ximemdup0 (keyword_start, p - keyword_start);
   if (*p  == '\0')
     return;
 
@@ -185,14 +184,14 @@ parse_line (char const *line, char **keyword, char **arg)
     continue;
   ++p;
 
-  *arg = xstrndup (arg_start, p - arg_start);
+  *arg = ximemdup0 (arg_start, p - arg_start);
 }
 
 /* FIXME: Write a string to standard out, while watching for "dangerous"
    sequences like unescaped : and = characters.  */
 
 static void
-append_quoted (const char *str)
+append_quoted (char const *str)
 {
   bool need_backslash = true;
 
@@ -236,7 +235,7 @@ append_quoted (const char *str)
    Return true if successful.  */
 
 static bool
-dc_parse_stream (FILE *fp, const char *filename)
+dc_parse_stream (FILE *fp, char const *filename)
 {
   size_t line_number = 0;
   char const *next_G_line = G_line;
@@ -254,7 +253,7 @@ dc_parse_stream (FILE *fp, const char *filename)
   if (term == NULL || *term == '\0')
     term = "none";
 
-  while (1)
+  while (true)
     {
       char *keywd, *arg;
       bool unrecognized;
@@ -371,7 +370,7 @@ dc_parse_stream (FILE *fp, const char *filename)
 }
 
 static bool
-dc_parse_file (const char *filename)
+dc_parse_file (char const *filename)
 {
   bool ok;
 
@@ -486,8 +485,8 @@ main (int argc, char **argv)
         {
           size_t len = obstack_object_size (&lsc_obstack);
           char *s = obstack_finish (&lsc_obstack);
-          const char *prefix;
-          const char *suffix;
+          char const *prefix;
+          char const *suffix;
 
           if (syntax == SHELL_SYNTAX_BOURNE)
             {
