@@ -1,5 +1,5 @@
 /* install - copy files and set attributes
-   Copyright (C) 1989-2020 Free Software Foundation, Inc.
+   Copyright (C) 1989-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <signal.h>
 #include <pwd.h>
 #include <grp.h>
-#include <selinux/selinux.h>
+#include <selinux/label.h>
 #include <sys/wait.h>
 
 #include "system.h"
@@ -69,10 +69,6 @@ static bool use_default_selinux_context = true;
 # define lchown(name, uid, gid) chown (name, uid, gid)
 #endif
 
-#if ! HAVE_MATCHPATHCON_INIT_PREFIX
-# define matchpathcon_init_prefix(a, p) /* empty */
-#endif
-
 /* The user name that will own the files, or NULL to make the owner
    the current user ID. */
 static char *owner_name;
@@ -175,7 +171,7 @@ extra_mode (mode_t input)
 
 /* Return true if copy of file SRC_NAME to file DEST_NAME is necessary. */
 static bool
-need_copy (const char *src_name, const char *dest_name,
+need_copy (char const *src_name, char const *dest_name,
            const struct cp_options *x)
 {
   struct stat src_sb, dest_sb;
@@ -268,7 +264,7 @@ cp_option_init (struct cp_options *x)
 {
   cp_options_default (x);
   x->copy_as_regular = true;
-  x->reflink_mode = REFLINK_NEVER;
+  x->reflink_mode = REFLINK_AUTO;
   x->dereference = DEREF_ALWAYS;
   x->unlink_dest_before_opening = true;
   x->unlink_dest_after_failed_open = false;
@@ -302,24 +298,37 @@ cp_option_init (struct cp_options *x)
   x->update = false;
   x->require_preserve_context = false;  /* Not used by install currently.  */
   x->preserve_security_context = false; /* Whether to copy context from src.  */
-  x->set_security_context = false;    /* Whether to set sys default context.  */
+  x->set_security_context = NULL;     /* Whether to set sys default context.  */
   x->preserve_xattr = false;
   x->verbose = false;
   x->dest_info = NULL;
   x->src_info = NULL;
 }
 
-#ifdef ENABLE_MATCHPATHCON
+static struct selabel_handle *
+get_labeling_handle (void)
+{
+  static bool initialized;
+  static struct selabel_handle *hnd;
+  if (!initialized)
+    {
+      initialized = true;
+      hnd = selabel_open (SELABEL_CTX_FILE, NULL, 0);
+      if (!hnd)
+        error (0, errno, _("warning: security labeling handle failed"));
+    }
+  return hnd;
+}
+
 /* Modify file context to match the specified policy.
    If an error occurs the file will remain with the default directory
-   context.  Note this sets the context to that returned by matchpathcon,
+   context.  Note this sets the context to that returned by selabel_lookup
    and thus discards MLS levels and user identity of the FILE.  */
 static void
 setdefaultfilecon (char const *file)
 {
   struct stat st;
   char *scontext = NULL;
-  static bool first_call = true;
 
   if (selinux_enabled != 1)
     {
@@ -329,51 +338,14 @@ setdefaultfilecon (char const *file)
   if (lstat (file, &st) != 0)
     return;
 
-  if (first_call && IS_ABSOLUTE_FILE_NAME (file))
+  struct selabel_handle *hnd = get_labeling_handle ();
+  if (!hnd)
+    return;
+  if (selabel_lookup (hnd, &scontext, file, st.st_mode) != 0)
     {
-      /* Calling matchpathcon_init_prefix (NULL, "/first_component/")
-         is an optimization to minimize the expense of the following
-         matchpathcon call.  Do it only once, just before the first
-         matchpathcon call.  We *could* call matchpathcon_fini after
-         the final matchpathcon call, but that's not necessary, since
-         by then we're about to exit, and besides, the buffers it
-         would free are still reachable.  */
-      char const *p0;
-      char const *p = file + 1;
-      while (ISSLASH (*p))
-        ++p;
-
-      /* Record final leading slash, for when FILE starts with two or more.  */
-      p0 = p - 1;
-
-      if (*p)
-        {
-          char *prefix;
-          do
-            {
-              ++p;
-            }
-          while (*p && !ISSLASH (*p));
-
-          prefix = malloc (p - p0 + 2);
-          if (prefix)
-            {
-              stpcpy (stpncpy (prefix, p0, p - p0), "/");
-              matchpathcon_init_prefix (NULL, prefix);
-              free (prefix);
-            }
-        }
-    }
-  first_call = false;
-
-  /* If there's an error determining the context, or it has none,
-     return to allow default context.  Note the "<<none>>" check
-     is only needed for libselinux < 1.20 (2005-01-04).  */
-  if ((matchpathcon (file, st.st_mode, &scontext) != 0)
-      || STREQ (scontext, "<<none>>"))
-    {
-      if (scontext != NULL)
-        freecon (scontext);
+      if (errno != ENOENT && ! ignorable_ctx_err (errno))
+        error (0, errno, _("warning: %s: context lookup failed"),
+               quotef (file));
       return;
     }
 
@@ -383,15 +355,7 @@ setdefaultfilecon (char const *file)
            quotef_n (0, file), quote_n (1, scontext));
 
   freecon (scontext);
-  return;
 }
-#else
-static void
-setdefaultfilecon (char const *file)
-{
-  (void) file;
-}
-#endif
 
 /* FILE is the last operand of this command.  Return true if FILE is a
    directory.  But report an error there is a problem accessing FILE,
@@ -431,7 +395,8 @@ static int
 make_ancestor (char const *dir, char const *component, void *options)
 {
   struct cp_options const *x = options;
-  if (x->set_security_context && defaultcon (component, S_IFDIR) < 0
+  if (x->set_security_context
+      && defaultcon (x->set_security_context, component, S_IFDIR) < 0
       && ! ignorable_ctx_err (errno))
     error (0, errno, _("failed to set default creation context for %s"),
            quoteaf (dir));
@@ -461,7 +426,7 @@ process_dir (char *dir, struct savewd *wd, void *options)
      and here we set the context for the final component. */
   if (ret == EXIT_SUCCESS && x->set_security_context)
     {
-      if (! restorecon (last_component (dir), false, false)
+      if (! restorecon (x->set_security_context, last_component (dir), false)
           && ! ignorable_ctx_err (errno))
         error (0, errno, _("failed to restore context for %s"),
                quoteaf (dir));
@@ -474,7 +439,7 @@ process_dir (char *dir, struct savewd *wd, void *options)
    Return true if successful.  */
 
 static bool
-copy_file (const char *from, const char *to, const struct cp_options *x)
+copy_file (char const *from, char const *to, const struct cp_options *x)
 {
   bool copy_into_self;
 
@@ -707,7 +672,7 @@ In the 4th form, create all components of the given DIRECTORY(ies).\n\
    Return true if successful.  */
 
 static bool
-install_file_in_file (const char *from, const char *to,
+install_file_in_file (char const *from, char const *to,
                       const struct cp_options *x)
 {
   struct stat from_sb;
@@ -788,10 +753,10 @@ install_file_in_file_parents (char const *from, char *to,
    Return true if successful.  */
 
 static bool
-install_file_in_dir (const char *from, const char *to_dir,
+install_file_in_dir (char const *from, char const *to_dir,
                      const struct cp_options *x, bool mkdir_and_install)
 {
-  const char *from_base = last_component (from);
+  char const *from_base = last_component (from);
   char *to = file_name_concat (to_dir, from_base, NULL);
   bool ret = true;
 
@@ -808,7 +773,7 @@ main (int argc, char **argv)
 {
   int optc;
   int exit_status = EXIT_SUCCESS;
-  const char *specified_mode = NULL;
+  char const *specified_mode = NULL;
   bool make_backups = false;
   char const *backup_suffix = NULL;
   char *version_control_string = NULL;
@@ -916,7 +881,7 @@ main (int argc, char **argv)
               /* Disable use of the install(1) specific setdefaultfilecon().
                  Note setdefaultfilecon() is different from the newer and more
                  generic restorecon() in that the former sets the context of
-                 the dest files to that returned by matchpathcon directly,
+                 the dest files to that returned by selabel_lookup directly,
                  thus discarding MLS level and user identity of the file.
                  TODO: consider removing setdefaultfilecon() in future.  */
               use_default_selinux_context = false;
@@ -924,7 +889,7 @@ main (int argc, char **argv)
               if (optarg)
                 scontext = optarg;
               else
-                x.set_security_context = true;
+                x.set_security_context = get_labeling_handle ();
             }
           else if (optarg)
             {
@@ -970,7 +935,7 @@ main (int argc, char **argv)
     die (EXIT_FAILURE, 0,
          _("cannot set target context and preserve it"));
 
-  if (scontext && setfscreatecon (se_const (scontext)) < 0)
+  if (scontext && setfscreatecon (scontext) < 0)
     die (EXIT_FAILURE, errno,
          _("failed to set default file creation context to %s"),
          quote (scontext));
@@ -1066,6 +1031,9 @@ main (int argc, char **argv)
             if (! install_file_in_dir (file[i], target_directory, &x,
                                        i == 0 && mkdir_and_install))
               exit_status = EXIT_FAILURE;
+#ifdef lint
+          dest_info_free (&x);
+#endif
         }
     }
 
