@@ -1,5 +1,5 @@
 /* mv -- move or rename files
-   Copyright (C) 1986-2020 Free Software Foundation, Inc.
+   Copyright (C) 1986-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include <assert.h>
-#include <selinux/selinux.h>
+#include <selinux/label.h>
 
 #include "system.h"
 #include "backupfile.h"
@@ -125,7 +125,7 @@ cp_option_init (struct cp_options *x)
   x->preserve_timestamps = true;
   x->explicit_no_preserve_mode= false;
   x->preserve_security_context = selinux_enabled;
-  x->set_security_context = false;
+  x->set_security_context = NULL;
   x->reduce_diagnostics = false;
   x->data_copy_required = true;
   x->require_preserve = false;  /* FIXME: maybe make this an option */
@@ -166,7 +166,7 @@ target_directory_operand (char const *file)
    Return true if successful.  */
 
 static bool
-do_move (const char *source, const char *dest, const struct cp_options *x)
+do_move (char const *source, char const *dest, const struct cp_options *x)
 {
   bool copy_into_self;
   bool rename_succeeded;
@@ -417,7 +417,9 @@ main (int argc, char **argv)
           if (selinux_enabled)
             {
               x.preserve_security_context = false;
-              x.set_security_context = true;
+              x.set_security_context = selabel_open (SELABEL_CTX_FILE, NULL, 0);
+              if (! x.set_security_context)
+                error (0, errno, _("warning: ignoring --context"));
             }
           break;
         case_GETOPT_HELP_CHAR;
@@ -511,6 +513,10 @@ main (int argc, char **argv)
           x.last_file = i + 1 == n_files;
           ok &= movefile (file[i], target_directory, true, &x);
         }
+
+#ifdef lint
+      dest_info_free (&x);
+#endif
     }
   else
     {
