@@ -1,5 +1,5 @@
 /* stat.c -- display file or file system status
-   Copyright (C) 2001-2020 Free Software Foundation, Inc.
+   Copyright (C) 2001-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -257,7 +257,7 @@ static char const *decimal_point;
 static size_t decimal_point_len;
 
 static bool
-print_stat (char *pformat, size_t prefix_len, unsigned int m,
+print_stat (char *pformat, size_t prefix_len, char mod, char m,
             int fd, char const *filename, void const *data);
 
 /* Return the type of the specified file system.
@@ -357,6 +357,8 @@ human_fstype (STRUCT_STATVFS const *statfsbuf)
       return "debugfs";
     case S_MAGIC_DEVFS: /* 0x1373 local */
       return "devfs";
+    case S_MAGIC_DEVMEM: /* 0x454D444D local */
+      return "devmem";
     case S_MAGIC_DEVPTS: /* 0x1CD1 local */
       return "devpts";
     case S_MAGIC_DMA_BUF: /* 0x444D4142 local */
@@ -369,6 +371,8 @@ human_fstype (STRUCT_STATVFS const *statfsbuf)
       return "efs";
     case S_MAGIC_EROFS_V1: /* 0xE0F5E1E2 local */
       return "erofs";
+    case S_MAGIC_EXFAT: /* 0x2011BAB0 local */
+      return "exfat";
     case S_MAGIC_EXFS: /* 0x45584653 local */
       return "exfs";
     case S_MAGIC_EXOFS: /* 0x5DF5 local */
@@ -499,6 +503,8 @@ human_fstype (STRUCT_STATVFS const *statfsbuf)
       return "rpc_pipefs";
     case S_MAGIC_SDCARDFS: /* 0x5DCA2DF5 local */
       return "sdcardfs";
+    case S_MAGIC_SECRETMEM: /* 0x5345434D local */
+      return "secretmem";
     case S_MAGIC_SECURITYFS: /* 0x73636673 local */
       return "securityfs";
     case S_MAGIC_SELINUX: /* 0xF97CFF8C local */
@@ -537,6 +543,8 @@ human_fstype (STRUCT_STATVFS const *statfsbuf)
       return "usbdevfs";
     case S_MAGIC_V9FS: /* 0x01021997 local */
       return "v9fs";
+    case S_MAGIC_VBOXSF: /* 0x786F4256 remote */
+      return "vboxsf";
     case S_MAGIC_VMHGFS: /* 0xBACBACBC remote */
       return "vmhgfs";
     case S_MAGIC_VXFS: /* 0xA501FCF5 remote */
@@ -559,6 +567,8 @@ human_fstype (STRUCT_STATVFS const *statfsbuf)
       return "z3fold";
     case S_MAGIC_ZFS: /* 0x2FC12FC1 local */
       return "zfs";
+    case S_MAGIC_ZONEFS: /* 0x5A4F4653 local */
+      return "zonefs";
     case S_MAGIC_ZSMALLOC: /* 0x58295829 local */
       return "zsmallocfs";
 
@@ -854,7 +864,7 @@ out_file_context (char *pformat, size_t prefix_len, char const *filename)
 
 /* Print statfs info.  Return zero upon success, nonzero upon failure.  */
 static bool ATTRIBUTE_WARN_UNUSED_RESULT
-print_statfs (char *pformat, size_t prefix_len, unsigned int m,
+print_statfs (char *pformat, size_t prefix_len, char mod _GL_UNUSED, char m,
               int fd, char const *filename,
               void const *data)
 {
@@ -1121,7 +1131,7 @@ format_code_offset (char const* directive)
    Return zero upon success, nonzero upon failure.  */
 static bool ATTRIBUTE_WARN_UNUSED_RESULT
 print_it (char const *format, int fd, char const *filename,
-          bool (*print_func) (char *, size_t, unsigned int,
+          bool (*print_func) (char *, size_t, char, char,
                               int, char const *, void const *),
           void const *data)
 {
@@ -1146,11 +1156,12 @@ print_it (char const *format, int fd, char const *filename,
         case '%':
           {
             size_t len = format_code_offset (b);
-            char const *fmt_char = b + len;
+            char fmt_char = *(b + len);
+            char mod_char = 0;
             memcpy (dest, b, len);
             b += len;
 
-            switch (*fmt_char)
+            switch (fmt_char)
               {
               case '\0':
                 --b;
@@ -1158,15 +1169,30 @@ print_it (char const *format, int fd, char const *filename,
               case '%':
                 if (1 < len)
                   {
-                    dest[len] = *fmt_char;
+                    dest[len] = fmt_char;
                     dest[len + 1] = '\0';
                     die (EXIT_FAILURE, 0, _("%s: invalid directive"),
                          quote (dest));
                   }
                 putchar ('%');
                 break;
+              case 'H':
+              case 'L':
+                mod_char = fmt_char;
+                fmt_char = *(b + 1);
+                if (print_func == print_stat
+                    && (fmt_char == 'd' || fmt_char == 'r'))
+                  {
+                    b++;
+                  }
+                else
+                  {
+                    fmt_char = mod_char;
+                    mod_char = 0;
+                  }
+                FALLTHROUGH;
               default:
-                fail |= print_func (dest, len, to_uchar (*fmt_char),
+                fail |= print_func (dest, len, mod_char, fmt_char,
                                     fd, filename, data);
                 break;
               }
@@ -1357,7 +1383,7 @@ do_stat (char const *filename, char const *format, char const *format2)
   int flags = 0;
   struct stat st;
   struct statx stx = { 0, };
-  const char *pathname = filename;
+  char const *pathname = filename;
   struct print_args pa;
   pa.st = &st;
   pa.btime = (struct timespec) {-1, -1};
@@ -1485,7 +1511,7 @@ do_stat (char const *filename, char const *format,
 
 /* Print stat info.  Return zero upon success, nonzero upon failure.  */
 static bool
-print_stat (char *pformat, size_t prefix_len, unsigned int m,
+print_stat (char *pformat, size_t prefix_len, char mod, char m,
             int fd, char const *filename, void const *data)
 {
   struct print_args *parg = (struct print_args *) data;
@@ -1517,7 +1543,12 @@ print_stat (char *pformat, size_t prefix_len, unsigned int m,
         }
       break;
     case 'd':
-      out_uint (pformat, prefix_len, statbuf->st_dev);
+      if (mod == 'H')
+        out_uint (pformat, prefix_len, major (statbuf->st_dev));
+      else if (mod == 'L')
+        out_uint (pformat, prefix_len, minor (statbuf->st_dev));
+      else
+        out_uint (pformat, prefix_len, statbuf->st_dev);
       break;
     case 'D':
       out_uint_x (pformat, prefix_len, statbuf->st_dev);
@@ -1562,6 +1593,17 @@ print_stat (char *pformat, size_t prefix_len, unsigned int m,
     case 's':
       out_int (pformat, prefix_len, statbuf->st_size);
       break;
+    case 'r':
+      if (mod == 'H')
+        out_uint (pformat, prefix_len, major (statbuf->st_rdev));
+      else if (mod == 'L')
+        out_uint (pformat, prefix_len, minor (statbuf->st_rdev));
+      else
+        out_uint (pformat, prefix_len, statbuf->st_rdev);
+      break;
+    case 'R':
+      out_uint_x (pformat, prefix_len, statbuf->st_rdev);
+      break;
     case 't':
       out_uint_x (pformat, prefix_len, major (statbuf->st_rdev));
       break;
@@ -1670,7 +1712,7 @@ default_format (bool fs, bool terse, bool device)
               /* TRANSLATORS: This string uses format specifiers from
                  'stat --help' without --file-system, and NOT from printf.  */
               format = xasprintf ("%s%s", format, _("\
-" "Device: %Dh/%dd\tInode: %-10i  Links: %-5h Device type: %t,%T\n\
+" "Device: %Hd,%Ld\tInode: %-10i  Links: %-5h Device type: %Hr,%Lr\n\
 "));
             }
           else
@@ -1678,7 +1720,7 @@ default_format (bool fs, bool terse, bool device)
               /* TRANSLATORS: This string uses format specifiers from
                  'stat --help' without --file-system, and NOT from printf.  */
               format = xasprintf ("%s%s", format, _("\
-" "Device: %Dh/%dd\tInode: %-10i  Links: %h\n\
+" "Device: %Hd,%Ld\tInode: %-10i  Links: %h\n\
 "));
             }
           free (temp);
@@ -1753,10 +1795,10 @@ Display file or file system status.\n\
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
 
       fputs (_("\n\
-The --cached MODE argument can be; always, never, or default.\n\
-`always` will use cached attributes if available, while\n\
-`never` will try to synchronize with the latest attributes, and\n\
-`default` will leave it up to the underlying file system.\n\
+The MODE argument of --cached can be: always, never, or default.\n\
+'always' will use cached attributes if available, while\n\
+'never' will try to synchronize with the latest attributes, and\n\
+'default' will leave it up to the underlying file system.\n\
 "), stdout);
 
       fputs (_("\n\
@@ -1769,8 +1811,10 @@ The valid format sequences for files (without --file-system):\n\
   %C   SELinux security context string\n\
 "), stdout);
       fputs (_("\
-  %d   device number in decimal\n\
-  %D   device number in hex\n\
+  %d   device number in decimal (st_dev)\n\
+  %D   device number in hex (st_dev)\n\
+  %Hd  major device number in decimal\n\
+  %Ld  minor device number in decimal\n\
   %f   raw mode in hex\n\
   %F   file type\n\
   %g   group ID of owner\n\
@@ -1784,6 +1828,10 @@ The valid format sequences for files (without --file-system):\n\
   %N   quoted file name with dereference if symbolic link\n\
   %o   optimal I/O transfer size hint\n\
   %s   total size, in bytes\n\
+  %r   device type in decimal (st_rdev)\n\
+  %R   device type in hex (st_rdev)\n\
+  %Hr  major device type in decimal, for character/block device special files\n\
+  %Lr  minor device type in decimal, for character/block device special files\n\
   %t   major device type in hex, for character/block device special files\n\
   %T   minor device type in hex, for character/block device special files\n\
 "), stdout);
